
#include <istream>
#include <ostream>

/*
  Note: for us to test that the code compiles we put empty
     implementations. For your solution pls add the implementations in a
     different file called SparseMatrix.inl and include this at the end of your
header file

*/

namespace MFound {

  template<class T>
  class SparseMatrix {
  public:
    
    // scalar type  (i.e. double)
    typedef typename T::Scalar Scalar;

    // array type: used to 
    typedef typename T::Array Array;


    // algebraic vector (column major)
    typedef typename T::Vector Vector;

    // constructor    
    // default constructor
    SparseMatrix();

    /* m - number of rows
       n - number of columns
       is_symmetric --> if this is true certrain operations can be optimized
                        so it should be stored  
    */
    SparseMatrix(int m, int n, bool is_symmetric = false);


    /* 
       standard copy contstructor
    */
    SparseMatrix(const SparseMatrix&);
        
    /* 
       standard assignment operator
    */
    SparseMatrix& operator=(const SparseMatrix&);


    // API

    /* 
       M(row, column)+=v;
       return false if row and column outside bounds
       if matrix is symmetric also does M(column, row)+=v
    */
    bool add(int row, int column, Scalar v);

    /*
      matrix vector multiplication: return M*v (v is a column vector)
      The implementtion can assume that the dimensions match.
    */
    Vector operator*(const Vector& v)const;

    /*
      matrix x matrix multplication
      returns M*m
      The implementtion can assume that the dimensions match.
      The implementation has to be efficient (O(n^2)) - assuming that each row
      has constant number of non-zero entries
    */
    SparseMatrix operator*(const SparseMatrix& m)const;        

  };// class

  /*
    vector matrix multiplication: return v*M (v is a row vector)
    The implementtion can assume that the dimensions match.
  */
  template<class T>
  typename T::Vector operator*(const typename T::Vector&, const SparseMatrix<T>&);
  
  // I/O operators: 
  template<class T>
  std::istream& operator>>(std::istream& f, SparseMatrix<T>&);
  
  template<class T>
  std::ostream& operator<<(std::ostream& f, const SparseMatrix<T>&);
  

  

};// namespace
